*******************************************************************************;
** SAS macro Lbl2Fmt  vs. 1.0.7d, 23 Jun 2011, by Jim Groeneveld, Netherlands *;
*******************************************************************************;
**                                                                            *;
*; %MACRO   LBL2FMT    /* Conversion of variable labels into varname format   */
/* -------------------------------------------------------------------------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  Data    =_LAST_     /* input data set                                    R */
,  Format  =$LBL2FMT   /* format name for resulting variable name format    R */
                       /* (character format specification without period)     */
,  VarList =           /* variable list to include, default: all variables  O */
,  CalledBy=           /* Macro name of macro or program for err0r report   O */
,  Version = 1.0.7d    /* forced version specification for version control  O */
,  Debug   = 0         /* Debug level; for testing, debugging purposes only O */
,  ForMyUse=           /* argument for the author's use only, do not use it ! */
)/ DES     =             'Convert variable labels into formats'   /*STORE*/    ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : LBL2FMT.SAS / LBL2FMT.107a                                    *;
%* Purpose    : conversion of variable labels into varname formats if any     *;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       : 23 Jun 2011                                                   *;
%* Version    : 1.0.7d                                                        *;
%*                                                                            *;
%* SAS version: 6.12 under Windows 95 / 8.2 under Windows 2K, XP              *;
%*                                                                            *;
%* Validation : - (not yet validated, check results, use at your own risk)    *;
%* Disclaimer : The author is not liable in any way for any negative, adverse *;
%*              consequences of the use or misuse of this program / macro.    *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Details:                                                                   *;
%* ========                                                                   *;
%* Specifying an applicable VarList causes the format to represent varname    *;
%* values as specified, i.e. any mixture of upper and lowercase.              *;
%* Not specifying a VarList causes the format to represent all varname values *;
%* as uppercase values. These would only match other UPPERCASE values.        *;
%*                                                                            *;
%* Output:                                                                    *;
%* =======                                                                    *;
%* - an active format that is defined as (see second example):                *;
%*        +-------------------------------+                                   *;
/*        |  PROC FORMAT; VALUE $CharFmt  |                                   */
/*        |  "VarA" = "Label of VarA"     |                                   */
/*        |  "VarB" = "Label of VarB"     |                                   */
/*        |  "VarC" = "Label of VarC"     |                                   */
/*        |  ; RUN;                       |                                   */
%*        +-------------------------------+                                   *;
%*                                                                            *;
%* Examples:                                                                  *;
%* =========                                                                  *;
/* 1. * Convert all occurring variable labels in dsn to a format with         */
/*    * uppercase variable name values;                                       */
/*    %LBL2FMT (Data=dsn);                                                    */
/*                                                                            */
/* 2. * Convert occurring variable labels for variables VarA, VarB and VarC   */
/*    * to the format $CharFmt for a character variable with specified-case   */
/*    * values 'VarA', 'VarB' and 'VarC';                                     */
/*    %LBL2FMT (Data=dsn, Format=$CharFmt, VarList=VarA VarB VarC)            */
%*                                                                            *;
%* Remarks:                                                                   *;
%* ========                                                                   *;
%* -                                                                          *;
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* = internal: - [none]                                                       *;
%* = attached: - [none]                                                       *;
%* = external: - ChkVar                                                       *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of LBL2FMT:                                                        *;
%* ====================                                                       *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0      10 Jun 1999  first working release, Q(uick)&D(irty)               *;
%* same     11 Jun 1999  minor improvements, added feedback, checks, comments *;
%* 0.1      18 Jun 1999  added RUN statement to generated FORMAT              *;
%* 0.1.1     2 Jul 1999  added code to compile but skip this macro further    *;
%* 0.1.2    20 Jul 1999  added %LOCAL statement for all local variables       *;
%* 0.1.3    28 Jul 1999  added code to delete the temporary scratch file,     *;
%*                       removed Version argument, added ForMyUse argument,   *;
%*                       adapted source code layout to answer GPP guidelines  *;
%* 1.0.0     2 Aug 1999  changed code to delete scratch file, removed NOXWAIT,*;
%*                       added use of environment variable TEMP for FileName, *;
%*                       added check for existing Data dataset,               *;
%*                       added check for existing variables in VarList,       *;
%* 1.0.1    17 Aug 1999  changed last check using the external macro ChkVar   *;
%* 1.0.2    17 Aug 1999  some very minor textual changes within comments      *;
%* 1.0.3    24 Sep 1999  changed 'ABORT ABEND' into only 'ABORT' with err0rs  *;
%* 1.0.4    28 Sep 1999  changed fixed temporary file name into pseudo-random *;
%* 1.0.5    30 Sep 1999  added check for existing auxiliary temporary file,   *;
%*                       removed FileName argument, now only automatically    *;
%* 1.0.6     5 Jul 2001  corrected bug with existing temp. unique file name   *;
%* 1.0.7    14 Apr 2004  added CalledBy argument, added Version control       *;
%*                       adapted PROC CONTENTS Name into UPCASE(Name) because *;
%*                          of SAS 8.2 backwards incompatibility with 6.12    *;
%*          14 Jul 2004  added format OTHER=[$CHAR32.] for unformatted length *;
%* 1.0.7a    5 Apr 2005  corrected processing of keyword _LAST_ with Data,    *;
%*                       added Debug control, added minor comment corrections *;
%* 1.0.7b   14 Jun 2005  corrected / activated code to delete scratch file,   *;
%*                       corrected bug with quote(s) in original label(s)     *;
%* 1.0.7c   21 Oct 2005  corrected bug caused by correcting previous bug:     *;
%*                         format with uppercase varnames into VarList spec.  *;
%*           2 Nov 2005  corrected code for TempDir values with ~ characters  *;
%*           7 Nov 2005  reset SYSLAST variable to original DATA variable     *;
%* 1.0.7d   23 Jun 2011  fixed scratch file deletion problem with FDELETE     *;
%*                                                                            *;
%* Future features:                                                           *;
%* ================                                                           *;
%* - (use of new macro UNIQUE to generate a random filename automatically)    *;
%* - use of PROC FORMAT CNTLIN and/or macro variable(s) instead of scratch    *;
%*      file to store generated format.                                       *;
%* - store state of options MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN    *;
%*      to reset them at their original state at the end if changed to debug. *;
%* - either use %SAS_name or code as in %MR2RM to check for valid SAS names.  *;
%*                                                                            *;
%******************************************************************************;

 %IF ( %UPCASE(&ForMyUse) EQ COMPILE ) %THEN %GOTO Finish;

%IF (&Debug EQ 1 OR &Debug GT 11) %THEN %DO;
  OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
%END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* LOCAL macro variables declarations;
%*___________________________________;

 %LOCAL    /* Define internal macro variables explicitely local */
  MacName  /* This macros external name: LBL2FMT */
  MacVs    /* Version number major.minor.patchSub of macro: 1.0.7d */
  MacDate  /* Date of this version of macro: 23 Jun 2011 */
  TempDir  /* Temporary directory name (from environment variable TEMP) */
  FileName /* temporary file name of format to write (for inclusion) */
  I        /* VarList element index */
  VarName  /* VarList element */
  Contents /* stand-in name for Contents dataset ___Cont_ */
  ErrCount /* number of erroneously specified arguments */
  Err      /* text 'E R R O R' as consecutive characters */
  Warn     /* text 'W A R N I N G' as consecutive characters */
  ;

  %LET MacName  = LBL2FMT ;
  %LET MacVs    = 1.0.7d;
  %LET MacDate  = 23 Jun 2011;
  %LET Contents = ___Cont_;
  %LET Err      = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn     = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

%* Macro header in LOG file, dependent on value of CalledBy;
 %IF ( %UPCASE(&CalledBy) EQ %UPCASE(&MacName) OR &CalledBy EQ ' ' ) %THEN
 %DO;
  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |   Macro &MacName, vs. &MacVs, by Jim Groeneveld, &MacDate   |;
  %PUT |     Conversion of variable labels into varname formats      |;
  %PUT |_____________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: Data set (to read) specification: &Data;
  %PUT &MacName: Name for resulting variable name format: &Format;
  %IF ( &VarList NE ) %THEN
    %PUT &MacName: Variable list: &VarList;
  %*PUT &MacName: ForMyUse: &ForMyUse;
 %END;

  %IF (&Version NE AND &Version NE &MacVs) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Specified version &Version does not match &MacVs,;
    %PUT .                       macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
    %GOTO Finish;
  %END;

%* Process keyword _LAST_ with Data;
  %IF (%UPCASE(&Data) EQ _LAST_) %THEN %LET Data = &SYSLAST;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Check arguments for legal contents and discrepancies ;
%*______________________________________________________;

  %LET ErrCount = 0;
  %IF ( &Data EQ ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Dataset not specified, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF ( %SYSFUNC(EXIST(&Data)) EQ 0 ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Data &Data dataset does not exist,;
    %PUT                      macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE
  %DO; %* Check for non-existing variables within dataset;
    %ChkVar (Data=&Data, VarList=&VarList, CalledBy=&MacName,
             Result = ErrCount); %* adding up non-existent variables ;
  %END;
  %IF ( &Format EQ ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Format name not specified, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %IF ( %SUBSTR(&Format,1,1) NE $ ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Format &Format does not start with $, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %IF ( &ErrCount GT 0 ) %THEN %GOTO Aborting;

%*~~~~~~~~~~~~~~~~~~~~;
%* Temporary FileName ;
%*____________________;

  %LET TempDir = %QUOTE(%SYSGET(TEMP));         %* Get contents of TEMP var;
  %IF ( &TempDir NE ) %THEN
  %DO;                                          %* If TEMP var with dir found;
    %IF ( %SUBSTR(&TempDir,%LENGTH(&TempDir),1) NE \ ) %THEN
    %DO;
      %LET TempDir = &TempDir.\;                %* Add trailing \ to dir;
    %END;
  %END;
%FmtFile:
  %LET FileName = ___%SYSFUNC(TIME());%* Add random time to FileName;
  %IF ( %SYSFUNC ( FILEEXIST(&TempDir.&FileName) ) ) %THEN GOTO FmtFile;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro actually starts here ;
%*____________________________;

PROC CONTENTS DATA=&Data OUT=&Contents NOPRINT;

* Determine max length of NAME and LABEL strings;
DATA _NULL_;
  IF 0 THEN SET &Contents;
  CALL SYMPUT ('LenName',  PUT(VLENGTH(NAME), BEST12.));
  CALL SYMPUT ('LenLabel', PUT(VLENGTH(LABEL),BEST12.));
  STOP;
RUN;
%*PUT LenName=&LenName, LenLabel=&LenLabel;

FILENAME scratch "&TempDir.&FileName";

DATA _NULL_;
  LENGTH Qname $ %EVAL(&LenName+2) Qlabel $ %EVAL(&LenLabel+2);
  SET &Contents (KEEP=Name Label) END=LastCase;     %* Use CONTENTS data;

  FILE scratch;                                    %* Open scratch file;

  IF ( _N_ = 1 ) THEN PUT "PROC FORMAT; VALUE &Format";  %* Initialize format;

 %IF ( &VarList NE ) %THEN
 %DO;                                              %* If VarList specified;
  %LET I = 1;                                      %* VarList element number I;
  %LET VarName = %SCAN(&VarList,&I,%STR( ));       %* Element I of VarList;
  %DO %WHILE (&VarName NE);                        %* For each element ...;

  %IF ( &I GT 1 ) %THEN ELSE;
    IF ( UPCASE(NAME)=UPCASE("&VarName") ) THEN
    DO;                                %* Output varnames as originally listed;
/*    IF ( Label NE ' ' ) THEN PUT """&VarName"" = '" Label +(-1) "'"; */
      Qname  = QUOTE(TRIM("&VarName")); Qlabel = QUOTE(TRIM(Label));
      IF ( Label NE ' ' ) THEN PUT Qname +(-1) '=' Qlabel;
    END;

  %LET I = %EVAL( &I + 1 );                        %* Increase sequence number;
  %LET VarName = %SCAN(&VarList,&I,%STR( ));       %* Next element of VarList;
  %END; %*=== While;                               %* Loop back to WHILE;

 %END; %* VarList;                                 %* All elements done;

 %ELSE
 %DO;                                              %* If VarList not specified;
%* Output all varnames CAPITALIZED, these should match uppercase name values;
/* IF ( Label NE ' ' ) THEN PUT '"' NAME +(-1) '" = "' Label +(-1) '"'; */
   Qname  = QUOTE(TRIM(NAME));  Qlabel = QUOTE(TRIM(Label));
   IF ( Label NE ' ' ) THEN PUT Qname +(-1) '=' Qlabel;
 %END;

  IF ( LastCase ) THEN PUT 'OTHER = [$CHAR32.]'
                           '; RUN;';               %* Close and RUN statement;

RUN;                                               %* Ready building format;

%INCLUDE scratch;                                  %* Make format effective;

%Aborting:

  DATA _NULL_;
    %IF (%UPCASE(&ForMyUse) NE KEEPTEMP) %THEN
      Dummy = FDELETE ("scratch");;                %* Delete scratch file;
    FILE LOG;
  RUN;
  FILENAME scratch CLEAR;;                         %* Break association;

  PROC DATASETS NOLIST;
    DELETE &Contents;                              %* Delete scratch dataset;
  RUN;

  %LET SYSLAST=&Data; %* reset to original SYSLAST value;
  %IF ( &ErrCount GT 0 ) %THEN
  %DO;
    DATA _NULL_;
      ABORT /*ABEND*/;
    RUN;
  %END;

%Finish:

%************;
%MEND LBL2FMT; * --------------------end-of-macro-Lbl2Fmt----------------------;
*************;
